@@ -2,10 +2,10 @@
 
 ;; Copyright (C) 1998 Free Software Foundation, Inc.
 
-;; Author: Oscar Figueiredo <Oscar.Figueiredo@epfl.ch>
-;; Maintainer: Oscar Figueiredo <Oscar.Figueiredo@epfl.ch>
+;; Author: Oscar Figueiredo <oscar@xemacs.org>
+;; Maintainer: Oscar Figueiredo <oscar@xemacs.org>
 ;; Created: Feb 1998
-;; Version: $Revision: 1.1 $
+;; Version: $Revision: 1.4 $
 ;; Keywords: help
 
 ;; This file is part of XEmacs
@@ -39,12 +39,17 @@
 (defcustom eudc-server nil
   "*The name or IP address of the directory server.
 A port number may be specified by appending a colon and a
-number to the name of the server."
+number to the name of the server. Use `localhost' if the directory
+server resides on your computer (BBDB backend)."
   :type  '(string :tag "Server")
   :group 'eudc)
 
+;; Known protocols (used in completion)
+;; Not to be mistaken with `eudc-supported-protocols'
+(defvar eudc-known-protocols '(bbdb ph ldap))
+
 (defvar eudc-supported-protocols nil
-  "Protocols currently supported by the Unified Directory Client.
+  "Protocols currently supported by EUDC.
 This variable is updated when protocol-specific libraries
 are loaded, *do not change by hand*.")
 
@@ -53,8 +58,8 @@ are loaded, *do not change by hand*.")
 Supported protocols are specified by `eudc-supported-protocols'."
   :type  `(choice :menu-tag "Protocol"
 		  ,@(mapcar (lambda (s) 
-			      (list 'string ':tag (symbol-name s)))
-			    eudc-supported-protocols))
+			      (list 'const ':tag (symbol-name s) s))
+			    eudc-known-protocols))
   :group 'eudc)
 
 
@@ -68,9 +73,12 @@ Supported protocols are specified by `eudc-supported-protocols'."
 If set to the symbol `all' return all attributes.
 nil means return the default attributes as configured in the server."
   :type  '(choice :menu-tag "Return Attributes"
-		  (const :menu-tag "Server Defaults (nil)" nil)
+		  (const :menu-tag "Server defaults (nil)" nil)
 		  (const :menu-tag "All" all)
-		  (repeat :menu-tag "Edit List" (symbol :tag "Field name")))
+		  (repeat :menu-tag "Attribute list" 
+			  :tag "Attribute name"
+			  :value (nil)
+			  (symbol :tag "Attribute name")))
   :group 'eudc)
 
 (defcustom eudc-multiple-match-handling-method 'select
@@ -81,11 +89,16 @@ Possible values are:
 `all' expand to all matches,
 `abort' the operation is aborted, an error is signaled."
   :type  '(choice :menu-tag "Method"
-		  (const :menu-tag "First"  first)
-		  (const :menu-tag "Select" select)
-		  (const :menu-tag "All"    all)
-		  (const :menu-tag "Abort"  abort)
-		  (const :menu-tag "None" nil))
+		  (const :menu-tag "Use First" 
+			 :tag "Use First"  first)
+		  (const :menu-tag "Select Interactively" 
+			 :tag "Select Interactively" select)
+		  (const :menu-tag "Use All" 
+			 :tag "Use All"    all)
+		  (const :menu-tag "Abort Operation" 
+			 :tag "Abort Operation"  abort)
+		  (const :menu-tag "Default (Use First)" 
+			 :tag "Default (Use First)" nil))
   :group 'eudc)
 
 (defcustom eudc-duplicate-attribute-handling-method '((email . duplicate))
@@ -114,15 +127,29 @@ different values."
 				       (const :menu-tag "First" first)
 				       (const :menu-tag "Concat" concat)
 				       (const :menu-tag "Duplicate" duplicate)))))
-  :group 'eudc
-  )
+  :group 'eudc)
 
-(defcustom eudc-inline-query-format '(firstname name)
+(defcustom eudc-inline-query-format '((name) 
+				      (firstname name))
   "*Format of an inline expansion query.
-If the inline query string consists of several words, this list specifies 
-how these individual words are mapped onto attribute names.
-If nil all the words will be mapped onto the default server/protocol key."
-  :type  '(repeat (symbol :tag "Attribute name"))
+This is a list of FORMATs.  A FORMAT is a list of one or more EUDC 
+attribute names.  A FORMAT applies if it contains as many attributes 
+as individual words in the inline query string.  
+If several FORMATs apply then they are tried in order until a match 
+is found.  If nil all the words will be mapped onto the default 
+server/protocol attribute name."
+  :tag "Format of Inline Expansion Queries"
+  :type  '(repeat
+	   (repeat
+	    :menu-tag "Format"
+	    :tag "Format"
+	    (choice
+	     :tag "Attribute"
+	     (const :menu-tag "First Name" :tag "First Name" firstname)
+	     (const :menu-tag "Surname" :tag "Surname" name)
+	     (const :menu-tag "Email Address" :tag "Email Address" email)
+	     (const :menu-tag "Phone" :tag "Phone" phone)
+	     (symbol :menu-tag "Other" :tag "Attribute name"))))
   :group 'eudc)
 
 (defcustom eudc-expansion-overwrites-query t
@@ -136,14 +163,22 @@ This variable controls what `eudc-expand-inline' actually inserts in the buffer.
 First element is a string passed to `format'.  Remaining elements are symbols
 indicating attribute names, the corresponding values are passed
 as additional arguments to `format'."
-  :type  '(list (string :tag "Format String")
-		(repeat :inline t
-			:tag "Attribute names"
-			(symbol :tag "")))
+  :type  '(list 
+	   (string :tag "Format String")
+	   (repeat :inline t
+		   :tag "Attributes"
+		   (choice 
+		    :tag "Attribute"
+		    (const :menu-tag "First Name" :tag "First Name" firstname)
+		    (const :menu-tag "Surname" :tag "Surname" name)
+		    (const :menu-tag "Email Address" :tag "Email Address" email)
+		    (const :menu-tag "Phone" :tag "Phone" phone)
+		    (symbol :menu-tag "Other")
+		    (symbol :tag "Attribute name"))))
   :group 'eudc)
 
 (defcustom eudc-inline-expansion-servers 'server-then-hotlist
-  "*Which servers to query for the expansion of inline queries.
+  "*Which servers to contact for the expansion of inline queries.
 Possible values are:
   `current-server': the EUDC current server
   `hotlist': the servers of the hotlist in the order they appear
@@ -174,7 +209,14 @@ If nil query all servers available from `eudc-inline-expansion-servers'"
 (defcustom eudc-query-form-attributes '(name firstname email phone)
   "*A list of attributes presented in the query form."
   :tag   "Attributes in Query Forms"
-  :type  '(repeat (symbol :tag "Attribute name"))
+  :type  '(repeat 	   
+	   (choice
+	    :tag "Attribute"
+	    (const :menu-tag "First Name" :tag "First Name" firstname)
+	    (const :menu-tag "Surname" :tag "Surname" name)
+	    (const :menu-tag "Email Address" :tag "Email Address" email)
+	    (const :menu-tag "Phone" :tag "Phone" phone)
+	    (symbol :menu-tag "Other" :tag "Attribute name")))
   :group 'eudc)
 
 (defcustom eudc-user-attribute-names-alist '((url . "URL")
@@ -205,7 +247,7 @@ at `_' signs and capitalizing the individual words."
   :tag   "User-defined Names of Directory Attributes"
   :type  '(repeat (cons :tag "Field"
 			(symbol :tag "Directory attribute")
-			(string :tag "User name")))
+			(string :tag "User friendly name ")))
   :group 'eudc)
 
 (defcustom eudc-use-raw-directory-names nil
@@ -215,13 +257,47 @@ names defined in `eudc-user-attribute-names-alist'."
   :type  'boolean
   :group 'eudc)
 
+(defcustom eudc-attribute-display-method-alist nil
+  "*An alist specifying methods to display attribute values.  Each member 
+of the list is of the form (NAME . FUNC) where NAME is a lowercased
+string naming a directory attribute (translated according to 
+`eudc-user-attribute-names-alist' if `eudc-use-raw-directory-names' is 
+non-nil) and FUNC a function that will be passed the corresponding 
+attribute values for display."
+  :tag "Attribute Decoding Functions"
+  :type '(repeat (cons :tag "Attribute"
+		       (symbol :tag "Name")
+		       (symbol :tag "Display Function")))
+  :group 'eudc)
+
+(defcustom eudc-external-viewers '(("XV" "xv" "-") 
+				   ("ImageMagick" "display" "-")
+				   ("ShowAudio" "showaudio"))
+  "*A list of viewer program specifications.
+Viewers are programs which can be piped a directory attribute value for
+display or arbitrary processing. Each specification is a list whose 
+first element is a string naming the viewer, the second element is the 
+executable program which should be invoked and following elements are
+arguments that should be passed to the program."
+  :tag "External Viewer Programs"
+  :type '(repeat (list :tag "Viewer"
+		       (string :tag "Name")
+		       (string :tag "Executable program")
+		       (repeat
+			:tag "Arguments"
+			:inline t
+			(string :tag "Argument"))))
+  :group 'eudc)
+
 (defcustom eudc-options-file "~/.eudc-options"
   "*A file where the `servers' hotlist is stored."
-  :type '(file :Tag "File Name:"))
+  :type '(file :Tag "File Name:")
+  :group 'eudc)
 
 (defcustom eudc-mode-hook nil
   "*Normal hook run on entry to EUDC mode."
-  :type '(repeat (sexp :tag "Hook")))
+  :type '(repeat (sexp :tag "Hook definition"))
+  :group 'eudc)
 
 ;;}}}
 
@@ -309,6 +385,12 @@ but influences the way records are displayed"
   :type 'boolean
   :group 'eudc-bbdb)
 
+(defcustom eudc-bbdb-enable-substring-matches t
+  "If non-nil, authorize substring matches in the same way BBDB does.
+Otherwise records must match queries exactly."
+  :type 'boolean
+  :group 'eudc-bbdb)
+
 ;;}}}
 
 
